// $Id: EXSConverter.m,v 1.2 2005/02/22 08:05:07 jeremy Exp $
//
// Copyright (c) 2005 Jeremy Cooper.  All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//    This product includes software developed by Jeremy Cooper.
// 4. The name of the author may not be used to endorse or promote products
//    derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
#include <sys/types.h>
#import "EXSConverter.h"
#import "exsutil.h"
#include <stdio.h>

@implementation EXSConverter

- (int) internalConvertFile: (FILE *) inFile forOutputAs: (FILE *) outFile
{
	char buf[512];
	size_t len;
	ssize_t nread;
	SInt32 numZones, numGroups, numSampleFiles;
	int i;
	
	//
	// First, read the file header.
	//
	if ((nread = fread(buf, 1, EXS_FILEINFO_SZ, inFile)) != EXS_FILEINFO_SZ)
		return -1;
	
	if (EXS_checkHeaderSync(buf, nread) != 0)
		return -1;
	if (EXS_fixHeader(buf, nread) != 0)
		return -1;

	//
	// Read the number of zones, groups and sample files, respectively, from
	// the header.
	//
	if (EXS_getSInt32(&numZones, &buf[0x58], nread - 0x58) != 0)
		return -1;
	if (EXS_getSInt32(&numGroups, &buf[0x5c], nread - 0x5c) != 0)
		return -1;
	if (EXS_getSInt32(&numSampleFiles, &buf[0x60], nread - 0x60) != 0)
		return -1;

	//
	// Write the fixed header to the output file (if given)
	//
	if (outFile != NULL && fwrite(buf, 1, nread, outFile) != nread)
		return -1;
			
	//
	// Convert all of the Zones.
	//
	for (i = 0; i < numZones; i++) {
		if ((nread = fread(buf, 1, EXS_ZONE_SZ, inFile)) != EXS_ZONE_SZ)
			return -1;
		if (EXS_checkHeaderSync(buf, nread) != 0)
			return -1;
		if (EXS_fixHeader(buf, nread) != 0)
			return -1;
		if (outFile != NULL && fwrite(buf, 1, nread, outFile) != nread)
			return -1;
	}

	//
	// Convert all of the Groups.
	//
	for (i = 0; i < numGroups; i++) {
		if ((nread = fread(buf, 1, EXS_GROUP_SZ, inFile)) != EXS_GROUP_SZ)
			return -1;
		if (EXS_checkHeaderSync(buf, nread) != 0)
			return -1;
		if (EXS_fixHeader(buf, nread) != 0)
			return -1;
		if (outFile != NULL && fwrite(buf, 1, nread, outFile) != nread)
			return -1;
	}
	
	//
	// Convert all of the Sample files.
	//
	for (i = 0; i < numSampleFiles; i++) {
		if ((nread = fread(buf, 1, EXS_SAMPLEFILE_SZ, inFile)) != EXS_SAMPLEFILE_SZ)
			return -1;
		if (EXS_checkHeaderSync(buf, nread) != 0)
			return -1;
		if (EXS_fixHeader(buf, nread) != 0)
			return -1;
		if (outFile != NULL && fwrite(buf, 1, nread, outFile) != nread)
			return -1;
	}
	
	//
	// Convert the Parameter block.
	//
	// The parameter block varies in size, but for our purpose, we only
	// care about reading and modifying its header.  The remainder of the block
	// takes up the rest of the file and doesn't need to be modified.
	//
	if ((nread = fread(buf, 1, EXS_OBJHDR_SZ, inFile)) != EXS_OBJHDR_SZ)
		return -1;
	if (EXS_checkHeaderSync(buf, nread) != 0)
		return -1;
	if (EXS_fixHeader(buf, nread) != 0)
		return -1;
	if (outFile != NULL && fwrite(buf, 1, nread, outFile) != nread)
		return -1;
	
	//
	// Copy the remainder of the file.
	//
	if (outFile != NULL)
		while ((nread = fread(buf, 1, sizeof(buf), inFile)) != 0)
			if (fwrite(buf, 1, nread, outFile) != nread)
				return -1;
	
	return 0;
}

- (int) internalConvert: (NSString *) input forOutputAs: (NSString *) output
{
	FILE *in = NULL, *out = NULL;
	const char *inFile = NULL, *outFile = NULL;
	int result = 0;
	
	inFile = [input UTF8String];
	if (output != nil)
		outFile = [output UTF8String];

	if ((in = fopen(inFile, "rb")) == NULL) {
		result = -1;
		goto done;
	}
	if (outFile != NULL && (out = fopen(outFile, "wb")) == NULL) {
		result = -1;
		goto done;
	}
	
	result = [self internalConvertFile: in forOutputAs: out];

done:
	if (in != NULL)
		fclose(in);
	if (out != NULL) {
		fclose(out);
		if (result != 0)
			unlink(outFile);
	}
	
	return result;
}

- (int) convert: (NSString *) input	forOutputAs: (NSString *) output
{
	return [self internalConvert: input forOutputAs: output];
}

- (int) fileIsOk: (NSString *) file
{
	return [self internalConvert: file forOutputAs: nil];
}

@end

